@@ -18,14 +18,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-0/engines/gob/videoplayer.cpp $
- * $Id: videoplayer.cpp 51921 2010-08-08 01:08:17Z drmccoy $
- *
  */
 
 
 #include "gob/videoplayer.h"
-#include "gob/helper.h"
 #include "gob/global.h"
 #include "gob/dataio.h"
 #include "gob/video.h"
@@ -38,15 +34,16 @@
 namespace Gob {
 
 VideoPlayer::Properties::Properties() : type(kVideoTypeTry), sprite(Draw::kFrontSurface),
-	x(-1), y(-1), width(-1), height(-1), flags(kFlagFrontSurface),
+	x(-1), y(-1), width(-1), height(-1), flags(kFlagFrontSurface), switchColorMode(false),
 	startFrame(-1), lastFrame(-1), endFrame(-1), forceSeek(false),
 	breakKey(kShortKeyEscape), palCmd(8), palStart(0), palEnd(255), palFrame(-1),
-	fade(false), waitEndFrame(true), canceled(false) {
+	noBlock(false), loop(false), fade(false), waitEndFrame(true),
+	hasSound(false), canceled(false) {
 
 }
 
 
-VideoPlayer::Video::Video() : decoder(0) {
+VideoPlayer::Video::Video() : decoder(0), live(false) {
 }
 
 bool VideoPlayer::Video::isEmpty() const {
@@ -59,10 +56,12 @@ void VideoPlayer::Video::close() {
 	decoder = 0;
 	fileName.clear();
 	surface.reset();
+
+	live = false;
 }
 
 
-const char *VideoPlayer::_extensions[] = { "IMD", "IMD", "VMD", "RMD", "SMD" };
+const char *const VideoPlayer::_extensions[] = { "IMD", "IMD", "VMD", "RMD", "SMD" };
 
 VideoPlayer::VideoPlayer(GobEngine *vm) : _vm(vm), _needBlit(false),
 	_noCursorSwitch(false), _woodruffCohCottWorkaround(false) {
@@ -86,6 +85,9 @@ void VideoPlayer::evaluateFlags(Properties &properties) {
 	} else {
 		properties.sprite = Draw::kBackSurface;
 	}
+
+	if (properties.noBlock && (properties.sprite == Draw::kFrontSurface))
+		properties.sprite = Draw::kBackSurface;
 }
 
 int VideoPlayer::openVideo(bool primary, const Common::String &file, Properties &properties) {
@@ -113,6 +115,16 @@ int VideoPlayer::openVideo(bool primary, const Common::String &file, Properties
 		if (!(video->decoder = openVideo(file, properties)))
 			return -1;
 
+		if (video->decoder->hasVideo() && !(properties.flags & kFlagNoVideo) &&
+		    (video->decoder->isPaletted() != !_vm->isTrueColor())) {
+			if (!properties.switchColorMode)
+				return -1;
+
+			_vm->setTrueColor(!video->decoder->isPaletted());
+
+			video->decoder->colorModeChanged();
+		}
+
 		// Set the filename
 		video->fileName = file;
 
@@ -143,42 +155,52 @@ int VideoPlayer::openVideo(bool primary, const Common::String &file, Properties
 
 			if (ownSurf) {
 				_vm->_draw->_spritesArray[properties.sprite] =
-					_vm->_video->initSurfDesc(_vm->_global->_videoMode,
-					                          screenSize ? _vm->_width  : video->decoder->getWidth(),
+					_vm->_video->initSurfDesc(screenSize ? _vm->_width  : video->decoder->getWidth(),
 					                          screenSize ? _vm->_height : video->decoder->getHeight(), 0);
 			}
 
-			if (!_vm->_draw->_spritesArray[properties.sprite]) {
+			if (!_vm->_draw->_spritesArray[properties.sprite] &&
+			    (properties.sprite != Draw::kFrontSurface) &&
+			    (properties.sprite != Draw::kBackSurface)) {
 				properties.sprite = -1;
 				video->surface.reset();
 				video->decoder->setSurfaceMemory();
-				video->decoder->setXY(0, 0);
+				properties.x = properties.y = 0;
 			} else {
 				video->surface = _vm->_draw->_spritesArray[properties.sprite];
-				video->decoder->setSurfaceMemory(video->surface->getVidMem(),
-						video->surface->getWidth(), video->surface->getHeight(), 1);
+				if (properties.sprite == Draw::kFrontSurface)
+					video->surface = _vm->_draw->_frontSurface;
+				if (properties.sprite == Draw::kBackSurface)
+					video->surface = _vm->_draw->_backSurface;
+
+				video->decoder->setSurfaceMemory(video->surface->getData(),
+						video->surface->getWidth(), video->surface->getHeight(), video->surface->getBPP());
 
 				if (!ownSurf || (ownSurf && screenSize)) {
-					if ((properties.x >= 0) || (properties.y >= 0))
-						video->decoder->setXY((properties.x < 0) ? 0xFFFF : properties.x,
-						                      (properties.y < 0) ? 0xFFFF : properties.y);
-					else
-						video->decoder->setXY();
+					if ((properties.x >= 0) || (properties.y >= 0)) {
+						properties.x = (properties.x < 0) ? 0xFFFF : properties.x;
+						properties.y = (properties.y < 0) ? 0xFFFF : properties.y;
+					} else
+						properties.x = properties.y = -1;
 				} else
-					video->decoder->setXY(0, 0);
+					properties.x = properties.y = 0;
 			}
 
 		} else {
 			properties.sprite = -1;
 			video->surface.reset();
 			video->decoder->setSurfaceMemory();
-			video->decoder->setXY(0, 0);
+			properties.x = properties.y = 0;
 		}
 	}
 
+	video->decoder->setXY(properties.x, properties.y);
+
 	if (primary)
 		_needBlit = (properties.flags & kFlagUseBackSurfaceContent) && (properties.sprite == Draw::kFrontSurface);
 
+	properties.hasSound = video->decoder->hasSound();
+
 	if (!video->decoder->hasSound())
 		video->decoder->setFrameRate(_vm->_util->getFrameRate());
 
@@ -196,6 +218,54 @@ bool VideoPlayer::closeVideo(int slot) {
 	return true;
 }
 
+void VideoPlayer::closeLiveSound() {
+	for (int i = 1; i < kVideoSlotCount; i++) {
+		Video *video = getVideoBySlot(i);
+		if (!video)
+			continue;
+
+		if (video->live)
+			closeVideo(i);
+	}
+}
+
+void VideoPlayer::closeAll() {
+	for (int i = 0; i < kVideoSlotCount; i++)
+		closeVideo(i);
+}
+
+void VideoPlayer::pauseVideo(int slot, bool pause) {
+	Video *video = getVideoBySlot(slot);
+	if (!video || !video->decoder)
+		return;
+
+	video->decoder->pauseVideo(pause);
+}
+
+void VideoPlayer::pauseAll(bool pause) {
+	for (int i = 0; i < kVideoSlotCount; i++)
+		pauseVideo(i, pause);
+}
+
+void VideoPlayer::finishVideoSound(int slot) {
+	Video *video = getVideoBySlot(slot);
+	if (!video || !video->decoder)
+		return;
+
+	video->decoder->finishSound();
+}
+
+void VideoPlayer::waitSoundEnd(int slot) {
+	Video *video = getVideoBySlot(slot);
+	if (!video || !video->decoder)
+		return;
+
+	video->decoder->finishSound();
+
+	while (video->decoder->isSoundPlaying())
+		_vm->_util->longDelay(1);
+}
+
 bool VideoPlayer::play(int slot, Properties &properties) {
 	Video *video = getVideoBySlot(slot);
 	if (!video)
@@ -203,9 +273,6 @@ bool VideoPlayer::play(int slot, Properties &properties) {
 
 	bool primary = slot == 0;
 
-	// NOTE: For testing (and comfort?) purposes, we enable aborting of all videos)
-	properties.breakKey = kShortKeyEscape;
-
 	if (properties.startFrame < 0)
 		properties.startFrame = video->decoder->getCurFrame() + 1;
 	if (properties.lastFrame  < 0)
@@ -230,6 +297,21 @@ bool VideoPlayer::play(int slot, Properties &properties) {
 
 	properties.canceled = false;
 
+	if (properties.noBlock) {
+		properties.waitEndFrame = false;
+
+		video->live       = true;
+		video->properties = properties;
+
+		updateLive(slot, true);
+		return true;
+	}
+
+	if ((_vm->getGameType() != kGameTypeUrban) && (_vm->getGameType() != kGameTypeBambou))
+		// NOTE: For testing (and comfort?) purposes, we enable aborting of all videos.
+		//       Except for Urban Runner and Bambou, where it leads to glitches
+		properties.breakKey = kShortKeyEscape;
+
 	while ((properties.startFrame != properties.lastFrame) &&
 	       (properties.startFrame < (int32)(video->decoder->getFrameCount() - 1))) {
 
@@ -260,8 +342,63 @@ void VideoPlayer::waitEndFrame(int slot, bool onlySound) {
 	if (!video)
 		return;
 
-	if (!onlySound || video->decoder->hasSound())
-		_vm->_util->delay(video->decoder->getTimeToNextFrame());
+	if (!onlySound || video->decoder->hasSound()) {
+		uint32 waitTime = video->decoder->getTimeToNextFrame();
+		if (!video->decoder->hasSound())
+			waitTime = video->decoder->getStaticTimeToNextFrame();
+
+		_vm->_util->delay(waitTime);
+	}
+}
+
+bool VideoPlayer::isPlayingLive() const {
+	const Video *video = getVideoBySlot(0);
+	return video && video->live;
+}
+
+void VideoPlayer::updateLive(bool force) {
+	for (int i = 0; i < kVideoSlotCount; i++)
+		updateLive(i, force);
+}
+
+void VideoPlayer::updateLive(int slot, bool force) {
+	Video *video = getVideoBySlot(slot);
+	if (!video || !video->live)
+		return;
+
+	if (video->properties.startFrame >= (int32)(video->decoder->getFrameCount() - 1)) {
+		// Video ended
+
+		if (!video->properties.loop) {
+			if (!(video->properties.flags & kFlagNoVideo))
+				WRITE_VAR_OFFSET(212, (uint32)-1);
+			_vm->_vidPlayer->closeVideo(slot);
+			return;
+		} else {
+			video->decoder->seek(0, SEEK_SET, true);
+			video->properties.startFrame = -1;
+		}
+	}
+
+	if (video->properties.startFrame == video->properties.lastFrame)
+		// Current video sequence ended
+		return;
+
+	if (!force && (video->decoder->getTimeToNextFrame() > 0))
+		return;
+
+	if (!(video->properties.flags & kFlagNoVideo))
+		WRITE_VAR_OFFSET(212, video->properties.startFrame + 1);
+
+	bool backwards = video->properties.startFrame > video->properties.lastFrame;
+	playFrame(slot, video->properties);
+
+	video->properties.startFrame += backwards ? -1 : 1;
+
+	if (video->properties.fade) {
+		_vm->_palAnim->fade(_vm->_global->_pPaletteDesc, -2, 0);
+		video->properties.fade = false;
+	}
 }
 
 bool VideoPlayer::playFrame(int slot, Properties &properties) {
@@ -326,7 +463,7 @@ bool VideoPlayer::playFrame(int slot, Properties &properties) {
 			_vm->_draw->forceBlit();
 	}
 
-	Graphics::Surface *surface = video->decoder->decodeNextFrame();
+	const Graphics::Surface *surface = video->decoder->decodeNextFrame();
 
 	WRITE_VAR(11, video->decoder->getCurFrame());
 
@@ -369,7 +506,8 @@ bool VideoPlayer::playFrame(int slot, Properties &properties) {
 
 			for (Common::List<Common::Rect>::const_iterator rect = dirtyRects.begin(); rect != dirtyRects.end(); ++rect)
 				_vm->_draw->invalidateRect(rect->left + ignoreBorder, rect->top, rect->right - 1, rect->bottom - 1);
-			_vm->_draw->blitInvalidated();
+			if (!video->live)
+				_vm->_draw->blitInvalidated();
 
 		} else if (video->surface == _vm->_draw->_frontSurface) {
 			for (Common::List<Common::Rect>::const_iterator rect = dirtyRects.begin(); rect != dirtyRects.end(); ++rect)
@@ -377,7 +515,7 @@ bool VideoPlayer::playFrame(int slot, Properties &properties) {
 
 		}
 
-		if ((video->decoder->getCurFrame() - 1) == properties.startFrame)
+		if (!video->live && ((video->decoder->getCurFrame() - 1) == properties.startFrame))
 			// Only retrace if we're playing the frame we actually want to play
 			_vm->_video->retrace();
 
@@ -414,7 +552,21 @@ void VideoPlayer::checkAbort(Video &video, Properties &properties) {
 				&_vm->_global->_inter_mouseY, &_vm->_game->_mouseButtons);
 
 		_vm->_inter->storeKey(_vm->_util->checkKey());
-		if (VAR(0) == (unsigned) properties.breakKey) {
+
+		// Check for that specific key
+		bool pressedBreak = (VAR(0) == (unsigned)properties.breakKey);
+
+		// Mouse buttons
+		if (properties.breakKey < 4)
+			if (_vm->_game->_mouseButtons & properties.breakKey)
+				pressedBreak = true;
+
+		// Any key
+		if (properties.breakKey == 4)
+			if (VAR(0) != 0)
+				pressedBreak = true;
+
+		if (pressedBreak) {
 			video.decoder->disableSound();
 
 			// Seek to the last frame. Some scripts depend on that.
@@ -501,7 +653,7 @@ bool VideoPlayer::hasEmbeddedFile(const Common::String &fileName, int slot) cons
 	return video->decoder->hasEmbeddedFile(fileName);
 }
 
-Common::MemoryReadStream *VideoPlayer::getEmbeddedFile(const Common::String &fileName, int slot) {
+Common::SeekableReadStream *VideoPlayer::getEmbeddedFile(const Common::String &fileName, int slot) {
 	const Video *video = getVideoBySlot(slot);
 	if (!video)
 		return 0;
@@ -553,9 +705,9 @@ void VideoPlayer::writeVideoInfo(const Common::String &file, int16 varX, int16 v
 	}
 }
 
-bool VideoPlayer::copyFrame(int slot, byte *dest,
-		uint16 left, uint16 top, uint16 width, uint16 height,
-		uint16 x, uint16 y, uint16 pitch, int16 transp) const {
+bool VideoPlayer::copyFrame(int slot, Surface &dest,
+		uint16 left, uint16 top, uint16 width, uint16 height, uint16 x, uint16 y,
+		int32 transp) const {
 
 	const Video *video = getVideoBySlot(slot);
 	if (!video)
@@ -565,51 +717,9 @@ bool VideoPlayer::copyFrame(int slot, byte *dest,
 	if (!surface)
 		return false;
 
-	int32 w = MIN<int32>(width , surface->w);
-	int32 h = MIN<int32>(height, surface->h);
-
-	const byte *src = (byte*)surface->pixels + (top * surface->pitch) + left;
-	      byte *dst =                   dest + (y   * pitch)          + x;
-
-	if (transp < 0) {
-		// No transparency
-
-		if ((x == 0) && (left == 0) && (pitch == surface->pitch) && (width == surface->w)) {
-			// Dimensions fit, we can copy everything at once
-
-			memcpy(dst, src, w * h);
-			return true;
-		}
-
-		// Copy row-by-row
-
-		while (h-- > 0) {
-			const byte *srcRow = src;
-						byte *dstRow = dst;
-
-			memcpy(dstRow, srcRow, w);
-
-			src += surface->pitch;
-			dst +=          pitch;
-		}
-
-		return true;
-	}
-
-	// Copy pixel-by-pixel
-
-	while (h-- > 0) {
-		const byte *srcRow = src;
-		      byte *dstRow = dst;
-
-		for (int32 i = 0; i < w; i++, srcRow++, dstRow++)
-			if (*srcRow != transp)
-				*dstRow = *srcRow;
-
-		src += surface->pitch;
-		dst +=          pitch;
-	}
+	Surface src(surface->w, surface->h, surface->format.bytesPerPixel, (byte *)surface->pixels);
 
+	dest.blit(src, left, top, left + width - 1, top + height - 1, x, y, transp);
 	return true;
 }
 
@@ -687,7 +797,7 @@ Common::String VideoPlayer::findFile(const Common::String &file, Properties &pro
 			if ((properties.type == kVideoTypeTry) || (properties.type == ((Type) i))) {
 				fileName = base + "." + _extensions[i];
 
-				if (_vm->_dataIO->existData(fileName.c_str())) {
+				if (_vm->_dataIO->hasFile(fileName)) {
 					properties.type = (Type) i;
 					break;
 				}
@@ -703,24 +813,24 @@ Common::String VideoPlayer::findFile(const Common::String &file, Properties &pro
 	return fileName;
 }
 
-Graphics::CoktelDecoder *VideoPlayer::openVideo(const Common::String &file, Properties &properties) {
+::Video::CoktelDecoder *VideoPlayer::openVideo(const Common::String &file, Properties &properties) {
 	Common::String fileName = findFile(file, properties);
 	if (fileName.empty())
 		return 0;
 
-	Common::SeekableReadStream *stream = _vm->_dataIO->getDataStream(fileName.c_str());
+	Common::SeekableReadStream *stream = _vm->_dataIO->getFile(fileName);
 	if (!stream)
 		return 0;
 
-	Graphics::CoktelDecoder *video = 0;
+	::Video::CoktelDecoder *video = 0;
 	if (properties.type == kVideoTypeIMD)
-		video = new Graphics::IMDDecoder(_vm->_mixer, Audio::Mixer::kSFXSoundType);
+		video = new ::Video::IMDDecoder(_vm->_mixer, Audio::Mixer::kSFXSoundType);
 	else if (properties.type == kVideoTypePreIMD)
-		video = new Graphics::PreIMDDecoder(properties.width, properties.height, _vm->_mixer, Audio::Mixer::kSFXSoundType);
+		video = new ::Video::PreIMDDecoder(properties.width, properties.height, _vm->_mixer, Audio::Mixer::kSFXSoundType);
 	else if (properties.type == kVideoTypeVMD)
-		video = new Graphics::VMDDecoder(_vm->_mixer, Audio::Mixer::kSFXSoundType);
+		video = new ::Video::VMDDecoder(_vm->_mixer, Audio::Mixer::kSFXSoundType);
 	else if (properties.type == kVideoTypeRMD)
-		video = new Graphics::VMDDecoder(_vm->_mixer, Audio::Mixer::kSFXSoundType);
+		video = new ::Video::VMDDecoder(_vm->_mixer, Audio::Mixer::kSFXSoundType);
 	else
 		warning("Couldn't open video \"%s\": Invalid video Type", fileName.c_str());
 
@@ -729,7 +839,7 @@ Graphics::CoktelDecoder *VideoPlayer::openVideo(const Common::String &file, Prop
 		return 0;
 	}
 
-	if (!video->load(stream)) {
+	if (!video->loadStream(stream)) {
 		delete video;
 		return 0;
 	}
@@ -741,7 +851,7 @@ Graphics::CoktelDecoder *VideoPlayer::openVideo(const Common::String &file, Prop
 }
 
 void VideoPlayer::copyPalette(const Video &video, int16 palStart, int16 palEnd) {
-	if (!video.decoder->hasPalette())
+	if (!video.decoder->hasPalette() || !video.decoder->isPaletted())
 		return;
 
 	if (palStart < 0)
