@@ -18,14 +18,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-0/engines/groovie/saveload.cpp $
- * $Id: saveload.cpp 48287 2010-03-18 15:54:40Z fingolfin $
- *
  */
 
 #include "groovie/saveload.h"
 
 #include "common/system.h"
+#include "common/substream.h"
 
 #define SUPPORTED_SAVEFILE_VERSION 1
 // 0 - Just script variables, compatible with the original
@@ -104,7 +102,7 @@ Common::InSaveFile *SaveLoad::openForLoading(const Common::String &target, int s
 	// Fill the SaveStateDescriptor if it was provided
 	if (descriptor) {
 		// Initialize the SaveStateDescriptor
-		descriptor->setVal("save_slot", Common::String('0' + slot));
+		descriptor->setSaveSlot(slot);
 		descriptor->setDeletableFlag(true);
 		descriptor->setWriteProtectedFlag(false);
 
@@ -134,7 +132,7 @@ Common::InSaveFile *SaveLoad::openForLoading(const Common::String &target, int s
 				description += c;
 			}
 		}
-		descriptor->setVal("description", description);
+		descriptor->setDescription(description);
 	}
 
 	// Return a substream, skipping the metadata
