@@ -18,12 +18,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-0/engines/m4/sprite.cpp $
- * $Id: sprite.cpp 50297 2010-06-26 00:36:43Z dreammaster $
- *
  */
 
 #include "common/rect.h"
+#include "common/textconsole.h"
 
 #include "m4/globals.h"
 #include "m4/graphics.h"
@@ -124,7 +122,7 @@ void M4Sprite::loadMadsSprite(Common::SeekableReadStream* source) {
 	bool spriteEnd = false;
 
 	// Set entire sprite contents to transparent pixels
-	fillRect(bounds(), TRANSPARENT_COLOUR_INDEX);
+	fillRect(bounds(), TRANSPARENT_COLOR_INDEX);
 
 	// Major line loop
 	for (int yp = 0; yp < h; ++yp) {
@@ -153,7 +151,7 @@ void M4Sprite::loadMadsSprite(Common::SeekableReadStream* source) {
 				byte v = source->readByte();
 				while (cmd-- > 0) {
 					if (x2 < w)
-						*destP++ = (v == 0xFD) ? TRANSPARENT_COLOUR_INDEX : v;
+						*destP++ = (v == 0xFD) ? TRANSPARENT_COLOR_INDEX : v;
 					++x2;
 				}
 			}
@@ -173,18 +171,18 @@ void M4Sprite::loadMadsSprite(Common::SeekableReadStream* source) {
 					byte v = source->readByte();
 					while (cmd-- > 0) {
 						if (x2 < w) {
-							*destP++ = (v == 0xFD) ? TRANSPARENT_COLOUR_INDEX : v;
+							*destP++ = (v == 0xFD) ? TRANSPARENT_COLOR_INDEX : v;
 						}
 						++x2;
 					}
 				} else {
 					// Handle writing out single pixel
-					*destP++ = (cmd == 0xFD) ? TRANSPARENT_COLOUR_INDEX : cmd;
+					*destP++ = (cmd == 0xFD) ? TRANSPARENT_COLOR_INDEX : cmd;
 					++x2;
 				}
 			}
 		}
-		
+
 		// Check if we need to scan forward to find the end of the line
 		if (!newLine) {
 			do {
@@ -203,7 +201,7 @@ void M4Sprite::loadMadsSprite(Common::SeekableReadStream* source) {
 }
 
 byte M4Sprite::getTransparencyIndex() const {
-		return TRANSPARENT_COLOUR_INDEX;
+		return TRANSPARENT_COLOR_INDEX;
 }
 
 } // End of namespace M4
