@@ -18,9 +18,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-0/engines/sci/engine/kernel.cpp $
- * $Id: kernel.cpp 52523 2010-09-04 14:46:29Z thebluegr $
- *
  */
 
 #include "sci/sci.h"
@@ -66,12 +63,12 @@ const Common::String &Kernel::getSelectorName(uint selector) {
 		//  We need this for proper workaround tables
 		// TODO: maybe check, if there is a fixed selector-table and error() out in that case
 		for (uint loopSelector = _selectorNames.size(); loopSelector <= selector; ++loopSelector)
-			_selectorNames.push_back(Common::String::printf("<noname%d>", loopSelector));
+			_selectorNames.push_back(Common::String::format("<noname%d>", loopSelector));
 	}
 
 	// Ensure that the selector has a name
 	if (_selectorNames[selector].empty())
-		_selectorNames[selector] = Common::String::printf("<noname%d>", selector);
+		_selectorNames[selector] = Common::String::format("<noname%d>", selector);
 
 	return _selectorNames[selector];
 }
@@ -88,13 +85,21 @@ const Common::String &Kernel::getKernelName(uint number) const {
 	return _kernelNames[number];
 }
 
+int Kernel::findKernelFuncPos(Common::String kernelFuncName) {
+	for (uint32 i = 0; i < _kernelNames.size(); i++)
+		if (_kernelNames[i] == kernelFuncName)
+			return i;
+
+	return -1;
+}
+
 int Kernel::findSelector(const char *selectorName) const {
 	for (uint pos = 0; pos < _selectorNames.size(); ++pos) {
 		if (_selectorNames[pos] == selectorName)
 			return pos;
 	}
 
-	debugC(2, kDebugLevelVM, "Could not map '%s' to any selector", selectorName);
+	debugC(kDebugLevelVM, "Could not map '%s' to any selector", selectorName);
 
 	return -1;
 }
@@ -103,6 +108,11 @@ void Kernel::loadSelectorNames() {
 	Resource *r = _resMan->findResource(ResourceId(kResourceTypeVocab, VOCAB_RESOURCE_SELECTORS), 0);
 	bool oldScriptHeader = (getSciVersion() == SCI_VERSION_0_EARLY);
 
+	// Starting with KQ7, Mac versions have a BE name table. GK1 Mac and earlier (and all
+	// other platforms) always use LE.
+	bool isBE = (g_sci->getPlatform() == Common::kPlatformMacintosh && getSciVersion() >= SCI_VERSION_2_1
+			&& g_sci->getGameId() != GID_GK1);
+
 	if (!r) { // No such resource?
 		// Check if we have a table for this game
 		// Some demos do not have a selector table
@@ -122,15 +132,15 @@ void Kernel::loadSelectorNames() {
 		return;
 	}
 
-	int count = READ_LE_UINT16(r->data) + 1; // Counter is slightly off
+	int count = isBE ? READ_BE_UINT16(r->data) : READ_LE_UINT16(r->data) + 1; // Counter is slightly off
 
 	for (int i = 0; i < count; i++) {
-		int offset = READ_LE_UINT16(r->data + 2 + i * 2);
-		int len = READ_LE_UINT16(r->data + offset);
+		int offset = isBE ? READ_BE_UINT16(r->data + 2 + i * 2) : READ_LE_UINT16(r->data + 2 + i * 2);
+		int len = isBE ? READ_BE_UINT16(r->data + offset) : READ_LE_UINT16(r->data + offset);
 
 		Common::String tmp((const char *)r->data + offset + 2, len);
 		_selectorNames.push_back(tmp);
-		//printf("%s\n", tmp.c_str());	// debug
+		//debug("%s", tmp.c_str());
 
 		// Early SCI versions used the LSB in the selector ID as a read/write
 		// toggle. To compensate for that, we add every selector name twice.
@@ -376,7 +386,6 @@ uint16 Kernel::findRegType(reg_t reg) {
 		break;
 	case SEG_TYPE_LOCALS:
 	case SEG_TYPE_STACK:
-	case SEG_TYPE_SYS_STRINGS:
 	case SEG_TYPE_DYNMEM:
 	case SEG_TYPE_HUNK:
 #ifdef ENABLE_SCI32
@@ -422,8 +431,8 @@ static void kernelSignatureDebugType(const uint16 type) {
 	while (list->typeCheck) {
 		if (type & list->typeCheck) {
 			if (!firstPrint)
-				printf(", ");
-			printf("%s", list->text);
+				debugN(", ");
+			debugN("%s", list->text);
 			firstPrint = false;
 		}
 		list++;
@@ -434,38 +443,38 @@ static void kernelSignatureDebugType(const uint16 type) {
 void Kernel::signatureDebug(const uint16 *sig, int argc, const reg_t *argv) {
 	int argnr = 0;
 	while (*sig || argc) {
-		printf("parameter %d: ", argnr++);
+		debugN("parameter %d: ", argnr++);
 		if (argc) {
 			reg_t parameter = *argv;
-			printf("%04x:%04x (", PRINT_REG(parameter));
+			debugN("%04x:%04x (", PRINT_REG(parameter));
 			int regType = findRegType(parameter);
 			if (regType)
 				kernelSignatureDebugType(regType);
 			else
-				printf("unknown type of %04x:%04x", PRINT_REG(parameter));
-			printf(")");
+				debugN("unknown type of %04x:%04x", PRINT_REG(parameter));
+			debugN(")");
 			argv++;
 			argc--;
 		} else {
-			printf("not passed");
+			debugN("not passed");
 		}
 		if (*sig) {
 			const uint16 signature = *sig;
 			if ((signature & SIG_MAYBE_ANY) == SIG_MAYBE_ANY) {
-				printf(", may be any");
+				debugN(", may be any");
 			} else {
-				printf(", should be ");
+				debugN(", should be ");
 				kernelSignatureDebugType(signature);
 			}
 			if (signature & SIG_IS_OPTIONAL)
-				printf(" (optional)");
+				debugN(" (optional)");
 			if (signature & SIG_NEEDS_MORE)
-				printf(" (needs more)");
+				debugN(" (needs more)");
 			if (signature & SIG_MORE_MAY_FOLLOW)
-				printf(" (more may follow)");
+				debugN(" (more may follow)");
 			sig++;
 		}
-		printf("\n");
+		debugN("\n");
 	}
 }
 
@@ -479,8 +488,15 @@ bool Kernel::signatureMatch(const uint16 *sig, int argc, const reg_t *argv) {
 		if ((type & SIG_IS_INVALID) && (!(curSig & SIG_IS_INVALID)))
 			return false; // pointer is invalid and signature doesn't allow that?
 
-		if (!((type & ~SIG_IS_INVALID) & curSig))
-			return false; // type mismatch
+		if (!((type & ~SIG_IS_INVALID) & curSig)) {
+			if ((type & ~SIG_IS_INVALID) == SIG_TYPE_ERROR && (curSig & SIG_IS_INVALID)) {
+				// Type is unknown (error - usually because of a deallocated object or
+				// stale pointer) and the signature allows invalid pointers. In this case,
+				// ignore the invalid pointer.
+			} else {
+				return false; // type mismatch
+			}
+		}
 
 		if (!(curSig & SIG_MORE_MAY_FOLLOW)) {
 			sig++;
@@ -521,6 +537,7 @@ void Kernel::mapFunctions() {
 
 	switch (g_sci->getPlatform()) {
 	case Common::kPlatformPC:
+	case Common::kPlatformFMTowns:
 		platformMask = SIGFOR_DOS;
 		break;
 	case Common::kPlatformPC98:
@@ -655,7 +672,7 @@ void Kernel::mapFunctions() {
 		}
 	} // for all functions requesting to be mapped
 
-	debugC(2, kDebugLevelVM, "Handled %d/%d kernel functions, mapping %d and ignoring %d.",
+	debugC(kDebugLevelVM, "Handled %d/%d kernel functions, mapping %d and ignoring %d.",
 				mapped + ignored, _kernelNames.size(), mapped, ignored);
 
 	return;
@@ -811,7 +828,8 @@ void Kernel::setDefaultKernelNames(GameFeatures *features) {
 enum {
 	kKernelEntriesSci2 = 0x8b,
 	kKernelEntriesGk2Demo = 0xa0,
-	kKernelEntriesSci21 = 0x9d
+	kKernelEntriesSci21 = 0x9d,
+	kKernelEntriesSci3 = 0xa1
 };
 
 void Kernel::setKernelNamesSci2() {
@@ -820,21 +838,30 @@ void Kernel::setKernelNamesSci2() {
 
 void Kernel::setKernelNamesSci21(GameFeatures *features) {
 	// Some SCI games use a modified SCI2 kernel table instead of the
-	// SCI2.1 kernel table. The GK2 demo does this as well as at least
-	// one version of KQ7 (1.4). We detect which version to use based on
+	// SCI2.1 kernel table. We detect which version to use based on
 	// how kDoSound is called from Sound::play().
+	// Known games that use this:
+	// GK2 demo
+	// KQ7 1.4
+	// PQ4 SWAT demo
+	// LSL6
+	// PQ4CD
+	// QFG4CD
 
 	// This is interesting because they all have the same interpreter
 	// version (2.100.002), yet they would not be compatible with other
 	// games of the same interpreter.
 
-	if (features->detectSci21KernelType() == SCI_VERSION_2) {
+	if (getSciVersion() != SCI_VERSION_3 && features->detectSci21KernelType() == SCI_VERSION_2) {
 		_kernelNames = Common::StringArray(sci2_default_knames, kKernelEntriesGk2Demo);
 		// OnMe is IsOnMe here, but they should be compatible
 		_kernelNames[0x23] = "Robot"; // Graph in SCI2
 		_kernelNames[0x2e] = "Priority"; // DisposeTextBitmap in SCI2
-	} else
+	} else if (getSciVersion() != SCI_VERSION_3) {
 		_kernelNames = Common::StringArray(sci21_default_knames, kKernelEntriesSci21);
+	} else if (getSciVersion() == SCI_VERSION_3) {
+		_kernelNames = Common::StringArray(sci21_default_knames, kKernelEntriesSci3);
+	}
 }
 
 #endif
@@ -903,6 +930,13 @@ void script_adjust_opcode_formats() {
 		g_opcode_formats[op_call][1] = Script_Word;
 		g_opcode_formats[op_callb][1] = Script_Word;
 	}
+
+	if (getSciVersion() >= SCI_VERSION_3) {
+		// TODO: There are also opcodes in
+		// here to get the superclass, and possibly the species too.
+		g_opcode_formats[0x4d/2][0] = Script_None;
+		g_opcode_formats[0x4e/2][0] = Script_None;
+	}
 #endif
 }
 
